VERSION 3.4
EXPORTS
    glutInit
    glutInitWindowPosition
    glutInitWindowSize
    glutInitDisplayMode
    glutInitDisplayString
    glutMainLoop
    glutMainLoopEvent
    glutLeaveMainLoop
    glutCreateWindow
    glutCreateSubWindow
    glutDestroyWindow
    glutSetWindow
    glutGetWindow
    glutSetWindowData
    glutGetWindowData
    glutSetWindowTitle
    glutSetIconTitle
    glutReshapeWindow
    glutPositionWindow
    glutShowWindow
    glutHideWindow
    glutIconifyWindow
    glutPushWindow
    glutPopWindow
    glutFullScreen
    glutPostWindowRedisplay
    glutPostRedisplay
    glutSwapBuffers
    glutWarpPointer
    glutSetCursor
    glutEstablishOverlay
    glutRemoveOverlay
    glutUseLayer
    glutPostOverlayRedisplay
    glutPostWindowOverlayRedisplay
    glutShowOverlay
    glutHideOverlay
    glutCreateMenu
    glutDestroyMenu
    glutGetMenu
    glutSetMenu
    glutGetMenuData
    glutSetMenuData
    glutAddMenuEntry
    glutAddSubMenu
    glutChangeToMenuEntry
    glutChangeToSubMenu
    glutRemoveMenuItem
    glutAttachMenu
    glutDetachMenu
    glutTimerFunc
    glutIdleFunc
    glutKeyboardFunc
    glutSpecialFunc
    glutReshapeFunc
    glutPositionFunc
    glutVisibilityFunc
    glutDisplayFunc
    glutMouseFunc
    glutMouseWheelFunc
    glutMotionFunc
    glutPassiveMotionFunc
    glutEntryFunc
    glutCloseFunc
    glutWMCloseFunc
    glutKeyboardUpFunc
    glutSpecialUpFunc
    glutJoystickFunc
    glutMenuStateFunc
    glutMenuStatusFunc
    glutMenuDestroyFunc
    glutOverlayDisplayFunc
    glutWindowStatusFunc
    glutSpaceballMotionFunc
    glutSpaceballRotateFunc
    glutSpaceballButtonFunc
    glutButtonBoxFunc
    glutDialsFunc
    glutTabletMotionFunc
    glutTabletButtonFunc
    glutSetOption
    glutGet
    glutDeviceGet
    glutGetModifiers
    glutLayerGet
    glutBitmapCharacter
    glutBitmapWidth
    glutStrokeCharacter
    glutStrokeWidth
    glutBitmapLength
    glutStrokeLength
    glutBitmapHeight
    glutStrokeHeight
    glutBitmapString
    glutStrokeString
    glutWireCube
    glutSolidCube
    glutWireSphere
    glutSolidSphere
    glutWireCone
    glutSolidCone
    glutWireTorus
    glutSolidTorus
    glutWireDodecahedron
    glutSolidDodecahedron
    glutWireOctahedron
    glutSolidOctahedron
    glutWireTetrahedron
    glutSolidTetrahedron
    glutWireIcosahedron
    glutSolidIcosahedron
    glutWireRhombicDodecahedron
    glutSolidRhombicDodecahedron
    glutWireSierpinskiSponge
    glutSolidSierpinskiSponge
    glutWireTeapot
    glutSolidTeapot
    glutWireTeacup
    glutSolidTeacup
    glutWireTeaspoon
    glutSolidTeaspoon
    glutWireCylinder
    glutSolidCylinder
    glutGameModeString
    glutEnterGameMode
    glutLeaveGameMode
    glutGameModeGet
    glutVideoResizeGet
    glutSetupVideoResizing
    glutStopVideoResizing
    glutVideoResize
    glutVideoPan
    glutSetColor
    glutGetColor
    glutCopyColormap
    glutIgnoreKeyRepeat
    glutSetKeyRepeat
    glutForceJoystickFunc
    glutExtensionSupported
    glutReportErrors
    glutGetProcAddress
    glutExit
    glutFullScreenToggle
    glutLeaveFullScreen
    glutSetMenuFont
    glutGetModeValues
    glutInitContextFlags
    glutInitContextVersion
    glutInitContextProfile
    glutInitErrorFunc
    glutInitWarningFunc
    __glutInitWithExit
    __glutCreateWindowWithExit
    __glutCreateMenuWithExit
    glutMultiButtonFunc
    glutMultiEntryFunc
    glutMultiMotionFunc
    glutMultiPassiveFunc
    glutInitContextFunc
    glutAppStatusFunc
    glutSetVertexAttribCoord3
    glutSetVertexAttribNormal
    glutSetVertexAttribTexCoord2
